/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *-----------------------------------------------------------------------------
 * $Author$
 * $Date$
 * $RCSfile: rtc_if.h,v $
 * $Revision$
 *
 * Description:
 *         Header file of RTC related interface.
 *---------------------------------------------------------------------------*/

/** @file rtc_if.h
 *  Header file of RTC related interface.
 */
 
#ifndef _RTC_IF_H_
#define _RTC_IF_H_

//----------------------------------------------------------------------------
// Constant definitions
//----------------------------------------------------------------------------
#include "x_typedef.h"
#include "sif_if.h"

//----------------------------------------------------------------------------
// Config
//----------------------------------------------------------------------------
#define ENABLE_UTC_BOUNDARY             1               ///< Flag to enable UTC boundary.

//----------------------------------------------------------------------------
// Constant definitions
//----------------------------------------------------------------------------
#define MAX_COUNT_YEAR                  99              ///< Max year count.

#if ENABLE_UTC_BOUNDARY
// UTC bounds
#define UTC_LOWER_BOUND                 946656000       ///< UTC lower bound: Jan  1 00:00:00, 2000.
//#define UTC_UPPER_BOUND                 2147483640  // Jan 19 11:14:00, 2038
#define UTC_UPPER_BOUND                 0xffffffffU     ///< UTC upper bound.
#define UTC_INIT_TIME                   946742400       ///< UTC initial time.
#endif /* ENABLE_UTC_BOUNDARY */

//-----------------------------------------------------------------------------
// Type definitions
//-----------------------------------------------------------------------------

/** RTC alarm handler function pointer 
 */
typedef void (*PFN_RTC_ALERT_HANDLER_T)(void);

/** RTC alram reset function pointer 
 */
typedef void (*PFN_RTC_ALERT_RESET_T)(void);

/** RTC date time structure
 */
typedef struct
{
    UINT8   u1Year;
    UINT8   u1Month;
    UINT8   u1Day;
    UINT8   u1Hour;
    UINT8   u1Minute;
    UINT8   u1Second;
    UINT8   u1Week;        
} RTC_T;

/** RTC function table
 */
typedef struct {
    BOOL (*pfnInit)(void);                                  ///< Pointer to initialization funtion.  
    BOOL (*pfnGetTimeDate)(UINT64*);                        ///< Pointer to get time funciton.            
    BOOL (*pfnSetTimeDate)(const UINT64*);                  ///< Pointer to set time function                                
    BOOL (*pfnGetAlertTime)(UINT64*);                       ///< Pointer to get alert time function
    BOOL (*pfnSetAlertTime)(UINT64);                        ///< Pointer to set alert time function    
    void (*pfnResetAlert)(void);                            ///< Pointer to alert reset function    
    void (*pfnHookAlertHandler)(PFN_RTC_ALERT_HANDLER_T);   ///< Pointer to alert handler hooking function. It is used to install alert handler.
    void (*pfnSetWakeup)(UINT32*);                          ///< Pointer to wakeup set function. It is called upon RTC wakeup enable
    BOOL (*pfnGetRtc)(RTC_T*);    
    BOOL (*pfnSetRtc)(RTC_T);       
} RTC_FuncTbl;

//----------------------------------------------------------------------------
// Static variables
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Public interface
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
/** RTC_BcdToBinary
 *  Convert a BCD to a binary
 *  @param u1Bcd The BCD to be converted
 *  @retval The converted binary
 */
//----------------------------------------------------------------------------
EXTERN UINT8 RTC_BcdToBinary(UINT8 u1Bcd);

//----------------------------------------------------------------------------
/** RTC_BinaryToBcd
 *  Convert a binary to a BCD
 *  @param u1Binary The binary to be converted
 *  @retval The converted BCD
 */
//----------------------------------------------------------------------------
EXTERN UINT8 RTC_BinaryToBcd(UINT8 u1Binary);

//----------------------------------------------------------------------------
/** RTC_RtcToUtc
 *  Translate RTC to UTC
 *  @param      prRtc   The RTC to be translated
 *  @param      pu8Utc  The translated UTC
 *  @retval     TRUE    Succeed
 *  @retval     FALSE   Fail
 */
//----------------------------------------------------------------------------
EXTERN BOOL RTC_RtcToUtc(UINT64* pu8Utc, const RTC_T* prRtc);


//----------------------------------------------------------------------------
/** RTC_UtcToRtc
 *  Translate UTC to RTC
 *  @param      u8Utc   The UTC to be translated
 *  @param      prRtc   The translated RTC
 *  @retval     TRUE    Succeed
 *  @retval     FALSE   Fail. 
 */
//----------------------------------------------------------------------------
EXTERN BOOL RTC_UtcToRtc(RTC_T* prRtc, UINT64 u8Utc);

//----------------------------------------------------------------------------
/** RTC_Init
 *  Initialize RTC driver and hardware
 *  @retval     TRUE    Succeed
 *  @retval     FALSE   Fail.
 */
//----------------------------------------------------------------------------
EXTERN BOOL RTC_Init(void);


//----------------------------------------------------------------------------
/** RTC_GetTimeDate
 *  Get time from RTC
 *  @param      prTime      Current time
 *  @retval     TRUE        Succeed
 *  @retval     FALSE       Fail.
 */
//----------------------------------------------------------------------------
EXTERN BOOL RTC_GetTimeDate(UINT64* prTime);


//----------------------------------------------------------------------------
/** RTC_SetTimeDate
 *  Set current time
 *  @param      prTime      The time to set
 *  @retval     TRUE        Succeed
 *  @retval     FALSE       Fail.
 */
//----------------------------------------------------------------------------
EXTERN BOOL RTC_SetTimeDate(const UINT64* prTime);

//----------------------------------------------------------------------------
/** RTC_SetAlertTime
 *  Set RTC alarm time
 *  @param      prTime      The alarm time to set
 *  @retval     TRUE        Succeed
 *  @retval     FALSE       Fail.
 */
//----------------------------------------------------------------------------
EXTERN BOOL RTC_SetAlertTime(UINT64 u8Utc);

//----------------------------------------------------------------------------
/** RTC_GetAlertTime
 *  Get alarm time set for RTC
 *  @param      prTime  The current alarm time setting
 *  @retval     TRUE    Succeed
 *  @retval     FALSE   Fail.
 */
//----------------------------------------------------------------------------
EXTERN BOOL RTC_GetAlertTime(UINT64* prTime);


//----------------------------------------------------------------------------
/** RTC_HookAlertHandler
 *  Hook the alert handler to RTC alarm interrupt
 *  @param      pfnHandler      The alert handler.
 *  @retval     none
 */
//----------------------------------------------------------------------------
EXTERN void RTC_HookAlertHandler(PFN_RTC_ALERT_HANDLER_T pfnHandler);

//----------------------------------------------------------------------------
/** RTC_SetWakeup
 *  Set RTC wakeup enable
 *  @param      pu4PowerDonwEn      The power down and wakeup enable setting.
 *  @retval     none
 */
//----------------------------------------------------------------------------
EXTERN void RTC_SetWakeup(UINT32* pu4PowerDownEn);

#endif
