/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_vbi_fltr.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * Description: 
 *         This header file contains public interfaces, constant and type 
 *         definitions of the vbi filter.
 *---------------------------------------------------------------------------*/

#ifndef _X_VBI_FLTR_H_
#define _X_VBI_FLTR_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "u_vbi_fltr.h"

#ifdef __cplusplus
extern "C" {
#endif

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------------
                    structures
 ----------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------------
                    functions
 ----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * Name: x_vbif_open
 *
 * Description: This API creates an instance of data filter dedicated to a specified 
 *              data type for a specified VBI source. Note that an instance of VBI 
 *              filter can only filter data with single type in a single VBI source.
 *              However, no limit is set on the number of filter instance for same
 *              data type in same VBI source. This makes multiple clients in the system 
 *              possible to acquire the same VBI data concurrently.
 *              
 *              When the opening process is finished, the notify function 
 *              (*x_vbif_nfy_fct) will be called with e_reason=VBIF_NFY_REASON_OPENED 
 *
 * Inputs:  e_fltr_type   Specifies the type of the data filter.
 *          pv_fltr_info  Contains the parameters for opening the specified data filter.
 *                        For example, if "e_fltr_type"=FLTR_TYPE_TTX_PKT, "pv_fltr_info"
 *                        shall point to a structure with type (FLTR_INIT_TTX_PKT_T *).
 *          e_src_type    Specifies the type of the VBI source.
 *          pv_src_info   Contains the parameters for opening the specified VBI source.
 *                        For example, if e_src_type == VBIF_SRC_TYPE_DIG_VBI_ES, 
 *                        pv_src_info should point to a structure 
 *                        of type (VBIF_SRC_DIG_VBI_ES_T *), which contains the parameters 
 *                        for opening the digital VBI elementary stream.
 *          e_fltr_mode   Specifies the filter mode, one of load, monitor, or Pump.
 *                           - VBIF_FLTR_MODE_LOAD : load the data, notify ONLY once and 
 *                                                   notify ONLY when data is first received.
 *                           - VBIF_FLTR_MODE_MONITOR : monitor the content of data, notify 
 *                                                      whenever data is changed
 *                           - VBIF_FLTR_MODE_PUMP : pump the data to client whenever data
 *                                                   is received.
 *          pv_nfy_tag    Contains the notify tag value.
 *          pf_nfy        References the notify function.
 *
 * Outputs: ph_filter     Contains a handle which references the newly created 
 *                        data filter instance.
 *          pe_cond       Contains the pointer to a variable to hold the newly created 
 *                        filter's condition. This argument can be NULL.
 *
 * Returns: VBIFR_OK                        Routine successful.
 *          VBIFR_INV_ARG                   Invalid arguments.
 *          VBIFR_NOT_SUPPORT_FLTR_TYPE     Filter type not supported.
 *          VBIFR_NOT_SUPPORT_SRC_TYPE      Source type not supported.
 *          VBIFR_NOT_SUPPORT_FLTR_MODE     Filter mode not supported.
 *          VBIFR_OUT_OF_MEMORY             Out of memory.
 *          VBIFR_INTERNAL_ERROR            Internal error.
 *          < 0                             Other error.
 ----------------------------------------------------------------------------*/
INT32 x_vbif_open(
    VBIF_FLTR_TYPE_T                e_fltr_type,
    const VOID*                     pv_fltr_info,
    VBIF_SRC_TYPE_T                 e_src_type,
    const VOID*                     pv_src_info,
    VBIF_FLTR_MODE_T                e_fltr_mode,
    const VOID*                     pv_nfy_tag,
    x_vbif_nfy_fct                  pf_nfy,
    HANDLE_T*                       ph_filter,
    VBIF_COND_T*                    pe_cond);

/*-----------------------------------------------------------------------------
 * Name: x_vbif_close
 *
 * Description: This API closes a data filter instance. No matter this API is called 
 *              in which condition, VBI filter will automatically close itself. 
 *              For example, if this API is called when VBI filter is in VBIF_COND_STARTED,
 *              VBI filter would then automatically go through VBIF_COND_STOPPING, 
 *              VBIF_COND_OPENED, VBIF_COND_CLOSING, and finally VBIF_COND_CLOSED.
 *              During the closing process, the notify function (*x_vbif_nfy_fct) will be 
 *              called with e_cond == <VBIF_COND_***> and e_reason == NFY_REASON_COND_CHANGED, 
 *              corresponding to each state change.
 *
 * Inputs:  h_filter  Specifies the handle of the data filter's instance.
 *
 * Outputs: pe_cond   Contains the condition after this API is called. 
 *                    *pe_cond would be undetermined if the function fails.
 *                    This argument can be NULL.
 *
 * Returns: VBIFR_OK                        Routine successful.
 *          VBIFR_INV_HANDLE                Invalid VBI filter handle.
 *          VBIFR_LOCK_ERROR                Internal lock error.
 *          VBIFR_MSG_Q_OVERFLOW            Message queue overflow.
 *          VBIFR_INTERNAL_ERROR            Internal error.
 *          < 0                             Other error.
 ----------------------------------------------------------------------------*/
INT32 x_vbif_close(
    HANDLE_T                        h_filter,
    VBIF_COND_T*                    pe_cond);

/*-----------------------------------------------------------------------------
 * Name: x_vbif_close_sync
 *
 * Description: This API synchronously closes a VBI filter instance. The behavior
 *              is similar to x_vbif_close () API except that the caller thread 
 *              would be blocked until the VBI filter instance finishes 
 *              the close process.
 *
 * Inputs:  h_filter  Specifies the handle of the data filter's instance.
 *
 * Outputs: pe_cond   Contains the condition after this API is called. 
 *                    If this API successfully returns, the condition shall always  
 *                    be VBIF_COND_CLOSED.
 *
 * Returns: VBIFR_OK                        Routine successful.
 *          VBIFR_INV_HANDLE                Invalid VBI filter handle.
 *          VBIFR_LOCK_ERROR                Internal lock error.
 *          VBIFR_MSG_Q_OVERFLOW            Message queue overflow.
 *          VBIFR_INTERNAL_ERROR            Internal error.
 *          < 0                             Other error.
 ----------------------------------------------------------------------------*/
INT32 x_vbif_close_sync (
    HANDLE_T                        h_filter,
    VBIF_COND_T*                    pe_cond);

/*-----------------------------------------------------------------------------
 * Name: x_vbif_start
 *
 * Description: This API starts data filtering. This API is effective ONLY when the 
 *              filter is in one of the following condition: 
 *                  - VBIF_COND_OPENING, 
 *                  - VBIF_COND_OPENED, 
 *                  - VBIF_COND_STARTING, 
 *                  - VBIF_COND_STOPPING 
 *              During the starting process, the notify function (*x_vbif_nfy_fct)
 *              will be called with e_cond == <VBIF_COND_***> 
 *              and e_reason == NFY_REASON_COND_CHANGED, 
 *              corresponding to each state change.
 *
 * Inputs:  h_filter  Specifies the handle of the filter's instance.
 *          
 * Outputs: pe_cond   Contains the condition after this API is called.
 *
 * Returns: VBIFR_OK                        Routine successful.
 *          VBIFR_INV_HANDLE                Invalid VBI filter handle.
 *          VBIFR_LOCK_ERROR                Internal lock error.
 *          VBIFR_MSG_Q_OVERFLOW            Message queue overflow.
 *          VBIFR_INTERNAL_ERROR            Internal error.
 *          < 0                             Other error.
 ----------------------------------------------------------------------------*/
INT32 x_vbif_start(
    HANDLE_T                        h_filter,
    VBIF_COND_T*                    pe_cond);

/*-----------------------------------------------------------------------------
 * Name: x_vbif_stop
 *
 * Description: This API stops data filtering. This API takes effect ONLY when 
 *              the filter is in VBIF_COND_STARTED condition. When the filter finishes 
 *              the stopping process, the notify function (*x_vbif_nfy_fct) will be 
 *              called with e_cond == VBIF_COND_OPENED and e_reason == NFY_REASON_COND_CHANGED.
 *
 * Inputs:  h_filter  Specifies the handle of the filter's instance.
 *          
 * Outputs: pe_cond   Contains the condition after this API is called.
 *
 * Returns: VBIFR_OK                        Routine successful.
 *          VBIFR_INV_HANDLE                Invalid VBI filter handle.
 *          VBIFR_LOCK_ERROR                Internal lock error.
 *          VBIFR_MSG_Q_OVERFLOW            Message queue overflow.
 *          VBIFR_INTERNAL_ERROR            Internal error.
 *          < 0                             Other error.
 ----------------------------------------------------------------------------*/
INT32 x_vbif_stop(
    HANDLE_T                        h_filter,
    VBIF_COND_T*                    pe_cond);

/*-----------------------------------------------------------------------------
 * Name: x_vbif_set
 *
 * Description: This API performs a set operation to the filter.
 *
 * Inputs:  h_filter        Specifies the handle of the filter's instance.
 *          e_set_type      Specifies the type of information which is
 *                          referenced by argument pv_set_info.
 *          pv_set_info     Contains the set information.
 *          z_set_info_len  Indicates the length of the data in the buffer
 *                          referenced by argument pv_set_info.
 *          
 * Outputs: pe_cond         Contains the condition after this API is called.
 *
 * Returns: VBIFR_OK                        Routine successful.
 *          VBIFR_INV_ARG                   Invalid arguments.
 *          VBIFR_INV_HANDLE                Invalid VBI filter handle.
 *          VBIFR_LOCK_ERROR                Internal lock error.
 *          VBIFR_MSG_Q_OVERFLOW            Message queue overflow.
 *          VBIFR_INTERNAL_ERROR            Internal error.
 *          < 0                             Other error.
 ----------------------------------------------------------------------------*/
INT32 x_vbif_set(
    HANDLE_T                        h_filter,
    VBIF_SET_TYPE_T                 e_set_type,
    const VOID*                     pv_set_info,
    SIZE_T                          z_set_info_len,
    VBIF_COND_T*                    pe_cond);


/*-----------------------------------------------------------------------------
 * Name: x_vbif_get
 *
 * Description: This API performs a get operation to the filter.
 *
 * Inputs:  h_filter         Specifies the handle of the filter's instance.
 *          e_get_type       Specifies the type of information which shall be
 *                           returned in the buffer, referenced by argument
 *                           pv_get_info.
 *          pz_get_info_len  Contains the length of the buffer which is
 *                           referenced by argument pv_get_info.
 *          
 * Outputs: pv_get_info      Contains the retrieved information.
 *          pz_get_info_len  Indicates the length of the valid data in the
 *                           buffer referenced by argument pv_get_info.
 *          pe_cond          Contains the condition or state of the filter.
 *
 * Returns: VBIFR_OK                        Routine successful.
 *          VBIFR_INV_ARG                   Invalid arguments.
 *          VBIFR_INV_HANDLE                Invalid VBI filter handle.
 *          VBIFR_LOCK_ERROR                Internal lock error.
 *          VBIFR_INTERNAL_ERROR            Internal error.
 *          < 0                             Other error.
 ----------------------------------------------------------------------------*/
INT32 x_vbif_get(
    HANDLE_T                        h_filter,
    VBIF_GET_TYPE_T                 e_get_type,
    VOID*                           pv_get_info,
    SIZE_T*                         pz_get_info_len,
    VBIF_COND_T*                    pe_cond);


#ifdef __cplusplus
}
#endif


#endif

