/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date: 2015/05/29 $
 * $RCSfile$
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/

/** @file pcie_if.c
 *  This C file implements the pcie initial hardware protocol functions.  
 */

//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------
#if defined(CC_MT5396) && defined(CC_MTK_LOADER)

#include "c_model.h"
#include "drvcust_if.h"

#include "x_typedef.h"
#include "x_hal_5381.h"
#include "x_bim.h"
#include "x_ckgen.h"
#include "x_gpio.h"
#include "x_printf.h"

//---------------------------------------------------------------------------
// Configurations
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Constant definitions
//---------------------------------------------------------------------------
#define PCIE_VIRT                      (IO_VIRT + 0x64000)

/*
 * PCIe PHY register.
 */
#define PCIE_PHY_BASE                    (PCIE_VIRT + 0x1500)
    #define PCIE_PHYD08                        (0x08)    /* PHY register. Tx amplitude */
    #define PCIE_LTSSM_OFFSET            (0x68)    /* PHY register. LTSSM register */
    #define PCIE_RESET_CTRL                 (0x64)    /* PHY register. RESET Control register */
    #define PCIE_TX_RX_PHASE_SEL     (0x5C)    /* PHY register. TX/RX phase select Control register */
    #define PCIE_PHYA90                        (0x90)    /* PHY register. CLK SSC on/off */
    #define PCIE_PHYA9C                        (0x9C)    /* PHY register. Tx pre-emphasis */
    #define PCIE_PHYAB4                        (0xB4)    /* PHY register. PLL calibration register */
    
/*
 * PCIe unit register offsets.
 */
#define PLDA_WDMA_PCI_ADDR0       (0x0000)
#define PLDA_WDMA_PCI_ADDR1       (0x0004)
#define PLDA_RDMA_PCI_ADDR0       (0x0020)
#define PLDA_RDMA_PCI_ADDR1       (0x0024)
    #define CONF_REG(r)                        (r & 0xFFC)
    #define CONF_BUS(b)                        ((b & 0xFF) << 24)
    #define CONF_DEV(d)                        ((d & 0x1F) << 19)
    #define CONF_FUNC(f)                       ((f & 0x7) << 16)

#define PLDA_WDMA_AHB_ADDR       (0x0008)
#define PLDA_RDMA_AHB_ADDR       (0x0028)

#define PLDA_WDMA_CONTROL          (0x000C)
#define PLDA_RDMA_CONTROL          (0x002C)
    #define CONF_SIZE                       ((4) << 12)
    #define CONF_BYTE1                    ((1) << 8)
    #define CONF_BYTE2                    ((3) << 8)    
    #define CONF_BYTE3                    ((7) << 8)    
    #define CONF_BYTE4                    ((0xF) << 8)    
    #define CONF_WCMD                     ((5) << 4)
    #define CONF_RCMD                      ((4) << 4)
    #define CONF_STOP                       ((1) << 1)
    #define CONF_START                     ((1) << 0)
    

#define PLDA_IMASK                         (0x0040)
#define PLDA_ISTATUS                     (0x0044)
#define PLDA_ICMD                           (0x0048)
#define PLDA_INFO                           (0x004C)
#define PLDA_IMSISTATUS               (0x0050)
    /* Write DMA */
    #define INT_WDMAEND                   (1 << 0)
    #define INT_WPCIERR                     (1 << 1)
    #define INT_WAHBERR                    (1 << 2)
    /* Read DMA */
    #define INT_RDMAEND                     (1 << 4)
    #define INT_RPCIERR                       (1 << 5)
    #define INT_RAHBERR                      (1 << 6)
    /* PCI-AHB window */
    #define INT_PAPOSTERR                       (1 << 8)
    #define INT_PAFETCHERR                     (1 << 9)
    #define INT_PAWIN0DB                         (1 << 10)
    #define INT_PAWIN1DB                         (1 << 11)   
    /* AHB-PCI window */
    #define INT_APDISCARD                       (1 << 13)
    #define INT_APFETCHERR                     (1 << 14)
    /* Legacy PCI Interrupt */    
    #define INT_INTA                            (1 << 16)
    #define INT_INTB                            (1 << 17)
    #define INT_INTC                            (1 << 18)    
    #define INT_INTD                            (1 << 19)
    /* MISC */        
    #define INT_SYSERR                       (1 << 20)
    #define INT_PMEVENT                     (1 << 21)
    #define INT_AER                              (1 << 22)
    #define INT_MSI                              (1 << 23)
    #define INT_PMSTATE                      (1 << 29)
    /* All error flag */
    #define INT_ALL_ERR                     (INT_WPCIERR |INT_WAHBERR |INT_RPCIERR|INT_RPCIERR| \
                                                              INT_PAPOSTERR|INT_PAFETCHERR|INT_APDISCARD|INT_APFETCHERR |\
                                                              INT_SYSERR)
    
#define PLDA_IMSIADDR                   (0x0054)

#define PLDA_SLOTCAP                      (0x005C)
#define PLDA_DV                                 (0x0060)
#define PLDA_SUB                               (0x0064)
#define PLDA_CREV                             (0x0068)
#define PLDA_SLOTCSR                       (0x006C)
#define PLDA_PRMCSR                        (0x0070)
#define PLDA_DEVCSR                        (0x0074)
#define PLDA_LINKCSR                       (0x0078)
#define PLDA_ROOTCSR                      (0x007C)

#define PLDA_PCIAHB0_BASE            (0x0080)
#define PLDA_PCIAHB1_BASE            (0x0084)

#define PLDA_AHBPCI0_BASE0            (0x0090)
#define PLDA_AHBPCI0_BASE1            (0x0094)
#define PLDA_AHBPCI1_BASE0            (0x0098)
#define PLDA_AHBPCI1_BASE1            (0x009C)

#define PLDA_AHBPCI_TIMER            (0x00A0)

#define PLDA_GEN2_CONF            (0x00A8)
#define PLDA_ASPM_CONF            (0x00AC)
#define PLDA_PM_STATUS            (0x00B0)
#define PLDA_PM_CONF0              (0x00B4)
#define PLDA_PM_CONF1             (0x00B8)
#define PLDA_PM_CONF2             (0x00BC)

// MSI communication address: 32 bits. Just use the msi register address as msi communication address.
// Must be 64 bits alignment, bit 0:2 = 000.
#define PLDA_MSI_ADDR_SETTING  (PCIE_VIRT + 0xC0)

//---------------------------------------------------------------------------
// Type definitions
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Macro definitions
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Imported variables
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Imported functions
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Static function forward declarations
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Static variables
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Static functions
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Inter-file functions
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Public functions
//---------------------------------------------------------------------------

//-------------------------------------------------------------------------
/** PCIE_PreInit
 *  Initialize pcie to set pcie_reset low
 *  @retval void
 */
//-------------------------------------------------------------------------
void PCIE_PreInit(void)
{
#ifdef PCIE_RESET_GPIO
    UINT32 regval32;
    UINT32 u4Gpio;
    UINT32 u4GpioPolarity;

    u4Gpio = PCIE_RESET_GPIO;
    u4GpioPolarity = 0;
    (void)GPIO_Output((INT32)u4Gpio, (INT32 *)&u4GpioPolarity);            

    Printf("PCIE_PreInit, reset gpio=%d.\n", u4Gpio);

    regval32 = HAL_READ32(PCIE_VIRT + PLDA_INFO);
    if (regval32 != 0x0152)
    {
        return;
    }

    /* Disable power down analog circuit  */
    regval32 = HAL_READ32(PCIE_PHY_BASE);
    regval32 &= ~0x20;
    HAL_WRITE32(PCIE_PHY_BASE, regval32);

    /* Enable SSC CLK  */
    regval32 = HAL_READ32(PCIE_PHY_BASE + PCIE_PHYA90);
    regval32 |= 0x800;
    HAL_WRITE32(PCIE_PHY_BASE + PCIE_PHYA90, regval32);

    /* Disable tx pre-emphasis  */
    regval32 = HAL_READ32(PCIE_PHY_BASE + PCIE_PHYA9C);
    regval32 &= ~0xF0;
    HAL_WRITE32(PCIE_PHY_BASE + PCIE_PHYA9C, regval32);

    /* Add tx amplitude  */
    regval32 = HAL_READ32(PCIE_PHY_BASE + PCIE_PHYD08);
    regval32 &= ~0x0F000000;
    regval32 |= 0x0A000000;    
    HAL_WRITE32(PCIE_PHY_BASE + PCIE_PHYD08, regval32);
#else
    Printf("No PCIE on this board.\n");
#endif    
}

//-------------------------------------------------------------------------
/** PCIE_PostInit
 *  Initialize pcie to set pcie_reset high
 *  @retval void
 */
//-------------------------------------------------------------------------
void PCIE_PostInit(void)
{
#ifdef PCIE_RESET_GPIO
    UINT32 regval32;
    UINT32 u4Gpio;
    UINT32 u4GpioPolarity;

    u4Gpio = PCIE_RESET_GPIO;
    u4GpioPolarity = 1;
    (void)GPIO_Output((INT32)u4Gpio, (INT32 *)&u4GpioPolarity);

    Printf("PCIE_PostInit, reset gpio=%d.\n", u4Gpio);

    regval32 = HAL_READ32(PCIE_PHY_BASE + PCIE_PHYAB4);
    if (regval32 & 0x3)
    {
        Printf("PCI-Express PLL calibration fail, status = 0x%04X.\n", regval32);
        return;
    }

    /* Active MAC/PHY */
    HAL_WRITE32(PCIE_PHY_BASE + PCIE_RESET_CTRL, 0xF);
#endif    
}

#endif /* #if defined(CC_MT5396) && defined(CC_MTK_LOADER) */

